/***********************************************************
 * $Id: POJOFieldComparator.java 20 2007-03-01 10:45:50Z wglas $
 * 
 * Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: Feb 28, 2007
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 * 
 ***********************************************************/

package org.clazzes.util.comparators;

import java.lang.reflect.InvocationTargetException;
import java.util.Comparator;

import org.clazzes.util.reflect.FieldAccessor;
import org.clazzes.util.reflect.POJOFieldAccessor;

/**
 * This comparator compares a POJO by the value of the supplied property.
 * 
 * @author wglas
 */
public class POJOFieldComparator implements Comparator {

    private FieldAccessor fieldAccessor;
    private Comparator fieldComparator;
    
    /**
     * Contructs a field comparator, which uses the given accessor and
     * Comparator.
     * 
     * @param fieldAccessor The field accessor, which is used to fetch field values.
     * @param fieldComparator The comparator, which compares field values.
     */
    public POJOFieldComparator(FieldAccessor fieldAccessor, Comparator fieldComparator)
    {
        this.fieldAccessor = fieldAccessor;
        this.fieldComparator = fieldComparator;
    }
    
    /**
     * Contructs a field comparator, which uses the given accessor and
     * constructs the comparator for the field using the given ComparatorFactory.
     * 
     * @param fieldAccessor The field accessor, which is used to fetch field values.
     * @param comparatorFactory The factory used to create a comparator base d on the type of the field.
     */
    public POJOFieldComparator(FieldAccessor fieldAccessor, ComparatorFactory comparatorFactory)
    {
        this.fieldAccessor = fieldAccessor;
        this.fieldComparator = comparatorFactory.getComparator(fieldAccessor.getType());
    }
    
    /**
     * Contructs a field comparator, which compare POJOs by the value
     * of the given property name using the given Comparator.
     * 
     * @param clazz The POJO clazz.
     * @param property The name of the POJO property, for which we use the getter function.
     * @param fieldComparator The underlying comparator.
     * @throws SecurityException If the security comtext does not allow acces to the getter.
     * @throws NoSuchMethodException If the geter method does not exist.
     * 
     * @see POJOFieldAccessor#POJOFieldAccessor(Class, String)
     */
    public POJOFieldComparator(Class clazz, String property, Comparator fieldComparator) throws SecurityException, NoSuchMethodException
    {
        this.fieldAccessor = new POJOFieldAccessor(clazz,property);
        this.fieldComparator = fieldComparator;        
    }
    
    /**
     * Contructs a field comparator, which comparea POJOs by the value
     * of the given property name and the comparator constructed using the given ComparatorFactory.
     * 
     * @param clazz The POJO clazz.
     * @param property The name of the POJO property, for which we use the getter function.
     * @param comparatorFactory The factory used to create a comparator base d on the type of the field.
     * @throws SecurityException If the security comtext does not allow acces to the getter.
     * @throws NoSuchMethodException If the geter method does not exist.
     * 
     * @see POJOFieldAccessor#POJOFieldAccessor(Class, String)
     */
    public POJOFieldComparator(Class clazz, String property, ComparatorFactory comparatorFactory) throws SecurityException, NoSuchMethodException
    {
        this.fieldAccessor = new POJOFieldAccessor(clazz,property);
        this.fieldComparator = comparatorFactory.getComparator(this.fieldAccessor.getType());   
    }
    
    /* (non-Javadoc)
     * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
     */
    public int compare(Object a, Object b) {
        
        if (a==null)
            return b==null ? 0 : -1;
        
        if (b==null)
            return 1;
        
        try {
            return this.fieldComparator.compare(this.fieldAccessor.getValueOf(a), this.fieldAccessor.getValueOf(b));
            
        } catch (IllegalArgumentException e) {
            throw new RuntimeException("Illegal argument fetching field ["+this.fieldAccessor.getName()+"] for comparison.",e);
        } catch (IllegalAccessException e) {
            throw new RuntimeException("Illegal access fetching field ["+this.fieldAccessor.getName()+"] for comparison.",e);
        } catch (InvocationTargetException e) {
            throw new RuntimeException("Caught exception fetching field ["+this.fieldAccessor.getName()+"] for comparison.",e);
        }
    }

    /**
     * @return the fieldAccessor
     */
    public FieldAccessor getFieldAccessor() {
        return this.fieldAccessor;
    }

    /**
     * @param fieldAccessor the fieldAccessor to set
     */
    public void setFieldAccessor(FieldAccessor fieldAccessor) {
        this.fieldAccessor = fieldAccessor;
    }

    /**
     * @return the fieldComparator
     */
    public Comparator getFieldComparator() {
        return this.fieldComparator;
    }

    /**
     * @param fieldComparator the fieldComparator to set
     */
    public void setFieldComparator(Comparator fieldComparator) {
        this.fieldComparator = fieldComparator;
    }

}
