/***********************************************************
 * $Id: ISO8601Format.java 39 2007-06-27 08:25:08Z wglas $
 * 
 * Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 2006-03-13
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 * 
 ***********************************************************/

package org.clazzes.util.datetime;

import java.text.FieldPosition;
import java.text.Format;
import java.text.ParsePosition;
import java.util.Calendar;
import java.util.TimeZone;

/**
 * Format Calendar objects cf. ISO8601
 * 
 * @author wglas
 */
public class ISO8601Format extends Format
{
    /**
     * To be changed upon class layout change.
     */
    private static final long serialVersionUID = 6403675010685159895L;

    /**
     * Format Calendars using only date fields.
     */
    public static int DATE_FORMAT = 1;
    
    /**
     * Format Calendars using date and time fields including the timezone designator.
     */
    public static int DATETIME_FORMAT = 2;
    
    /**
     * Format Calendars using date and time fields up to milliseconds including the timezone designator.
     */
    public static int MILLISECOND_FORMAT = 3;
    
    private int format;
    
    /**
     * @param format {@link ISO8601Format#DATE_FORMAT} or 
     *               {@link ISO8601Format#DATETIME_FORMAT} or
     *               {@link ISO8601Format#MILLISECOND_FORMAT}.
     */
    public ISO8601Format(int format)
    {
        this.format = format;
    }
    
    
    /* (non-Javadoc)
     * @see java.text.Format#format(java.lang.Object, java.lang.StringBuffer, java.text.FieldPosition)
     */
    @Override
    public StringBuffer format(Object o, StringBuffer buf,
            FieldPosition pos)
    {
        if (o instanceof UtcTimestamp)
            return buf.append(((UtcTimestamp)o).toString(this.format));
        
        Calendar c = (Calendar)o;
        
        if (this.format == DATE_FORMAT)
            buf.append(String.format("%04d-%02d-%02d",
                    c.get(Calendar.YEAR),
                    c.get(Calendar.MONTH)+1,
                    c.get(Calendar.DAY_OF_MONTH)
            ));
        else if (this.format == DATETIME_FORMAT)
        {
            int off = c.get(Calendar.ZONE_OFFSET) + c.get(Calendar.DST_OFFSET);
            
            if (off == 0)
                buf.append(String.format("%04d-%02d-%02dT%02d:%02d:%02dZ",
                        c.get(Calendar.YEAR),
                        c.get(Calendar.MONTH)+1,
                        c.get(Calendar.DAY_OF_MONTH),
                        c.get(Calendar.HOUR_OF_DAY),
                        c.get(Calendar.MINUTE),
                        c.get(Calendar.SECOND)
                ));
            else if (off > 0)
                buf.append(String.format("%04d-%02d-%02dT%02d:%02d:%02d+%02d%02d",
                        c.get(Calendar.YEAR),
                        c.get(Calendar.MONTH)+1,
                        c.get(Calendar.DAY_OF_MONTH),
                        c.get(Calendar.HOUR_OF_DAY),
                        c.get(Calendar.MINUTE),
                        c.get(Calendar.SECOND),
                        off / 3600000, (off / 60000) % 60
                ));
            else
                buf.append(String.format("%04d-%02d-%02dT%02d:%02d:%02d-%02d%02d",
                        c.get(Calendar.YEAR),
                        c.get(Calendar.MONTH)+1,
                        c.get(Calendar.DAY_OF_MONTH),
                        c.get(Calendar.HOUR_OF_DAY),
                        c.get(Calendar.MINUTE),
                        c.get(Calendar.SECOND),
                        off / -3600000, (off / -60000) % 60
                ));
               
        }
        else
        {
            int off = c.get(Calendar.ZONE_OFFSET) + c.get(Calendar.DST_OFFSET);
            
            if (off == 0)
                buf.append(String.format("%04d-%02d-%02dT%02d:%02d:%02d.%03dZ",
                        c.get(Calendar.YEAR),
                        c.get(Calendar.MONTH)+1,
                        c.get(Calendar.DAY_OF_MONTH),
                        c.get(Calendar.HOUR_OF_DAY),
                        c.get(Calendar.MINUTE),
                        c.get(Calendar.SECOND),
                        c.get(Calendar.MILLISECOND)
                ));
            else if (off > 0)
                buf.append(String.format("%04d-%02d-%02dT%02d:%02d:%02d.%03d+%02d%02d",
                        c.get(Calendar.YEAR),
                        c.get(Calendar.MONTH)+1,
                        c.get(Calendar.DAY_OF_MONTH),
                        c.get(Calendar.HOUR_OF_DAY),
                        c.get(Calendar.MINUTE),
                        c.get(Calendar.SECOND),
                        c.get(Calendar.MILLISECOND),
                        off / 3600000, (off / 60000) % 60
                ));
            else
                buf.append(String.format("%04d-%02d-%02dT%02d:%02d:%02d.%03d-%02d%02d",
                        c.get(Calendar.YEAR),
                        c.get(Calendar.MONTH)+1,
                        c.get(Calendar.DAY_OF_MONTH),
                        c.get(Calendar.HOUR_OF_DAY),
                        c.get(Calendar.MINUTE),
                        c.get(Calendar.SECOND),
                        c.get(Calendar.MILLISECOND),
                        off / -3600000, (off / -60000) % 60
                ));
               
        }
        
        return buf;
    }

    static private int extractInt(String str, ParsePosition pos, int nDigit)
    {
       int ret = 0;
        
       int idx = pos.getIndex();
       int iDigit = 0;
       
       while (iDigit < nDigit && Character.isDigit(str.charAt(idx)))
        {
            ret = 10*ret + Character.digit(str.charAt(idx), 10);
            ++idx;
            ++iDigit;
        }
        
        if (iDigit < nDigit) { pos.setErrorIndex(idx); return -1; }
        
        pos.setIndex(idx);
        return ret;
    }
    
    /* (non-Javadoc)
     * @see java.text.Format#parseObject(java.lang.String, java.text.ParsePosition)
     */
    @Override
    public Object parseObject(String str, ParsePosition pos)
    {
        int oidx = pos.getIndex();
        
        int year = extractInt(str,pos,4);
        
        if (year < 0) { pos.setIndex(oidx); return null; }
        if (str.charAt(pos.getIndex()) != '-') { pos.setErrorIndex(pos.getIndex()); pos.setIndex(oidx); return null; }
        pos.setIndex(pos.getIndex()+1);
        
        int month = extractInt(str,pos,2);
        if (month < 0) { pos.setIndex(oidx); return null; }
        if (str.charAt(pos.getIndex()) != '-') { pos.setErrorIndex(pos.getIndex()); pos.setIndex(oidx); return null; }
        pos.setIndex(pos.getIndex()+1);
        
        int day = extractInt(str,pos,2);
        if (day < 0) { pos.setIndex(oidx); return null; }
        Calendar c = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
        c.set(year, month-1, day);

        if (pos.getIndex() >= str.length() || str.charAt(pos.getIndex()) != 'T')
        {
            c.set(Calendar.HOUR_OF_DAY, 0);
            c.set(Calendar.MINUTE, 0);
            c.set(Calendar.SECOND, 0);
            c.set(Calendar.MILLISECOND, 0);
            return c;
        }
        
        pos.setIndex(pos.getIndex()+1);
        
        int hour = extractInt(str,pos,2);
        if (hour < 0) { pos.setIndex(oidx); return null; }
        if (str.charAt(pos.getIndex()) != ':') { pos.setErrorIndex(pos.getIndex()); pos.setIndex(oidx); return null; }
        pos.setIndex(pos.getIndex()+1);
      
        int minute = extractInt(str,pos,2);
        if (minute < 0) { pos.setIndex(oidx); return null; }
        if (str.charAt(pos.getIndex()) != ':') { pos.setErrorIndex(pos.getIndex()); pos.setIndex(oidx); return null; }
        pos.setIndex(pos.getIndex()+1);
      
        int second = extractInt(str,pos,2);
        if (second < 0) { pos.setIndex(oidx); return null; }
      
        c.set(Calendar.HOUR_OF_DAY, hour);
        c.set(Calendar.MINUTE, minute);
        c.set(Calendar.SECOND, second);
        
        int millis;
            
        if (str.charAt(pos.getIndex()) == '.')
        {
            pos.setIndex(pos.getIndex()+1);
            
            millis = extractInt(str,pos,3);
            if (millis < 0) { pos.setIndex(oidx); return null; }
        }
        else
            millis = 0;
        
        c.set(Calendar.MILLISECOND, millis);
        
        if (pos.getIndex() >= str.length()) return c;
        
        if (str.charAt(pos.getIndex()) == 'Z')
        {
            pos.setIndex(pos.getIndex()+1);
            return c;
        }
        
        int sign = 0;
        
        switch (str.charAt(pos.getIndex()))
        {
        case '+':
            sign = 1;
            break;
        case '-':
            sign = -1;
            break;
        default:
            return c;
        }
       
        pos.setIndex(pos.getIndex()+1);

        int tz_hour = extractInt(str,pos,2);
        if (tz_hour < 0) { pos.setIndex(oidx); return null; }
         
        int tz_minute = extractInt(str,pos,2);
        if (tz_hour < 0) { pos.setIndex(oidx); return null; }
         
        int tz_off = sign * ( tz_hour * 3600000 + tz_minute * 60000 );
        
        c.set(Calendar.ZONE_OFFSET,tz_off);
        return c;
    }

}
