/***********************************************************
 * $Id: FieldAccessor.java 57 2007-08-13 13:46:49Z rbreuss $
 * 
 * Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 2006-03-13
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 * 
 ***********************************************************/

package org.clazzes.util.reflect;

import java.lang.annotation.Annotation;
import java.lang.reflect.InvocationTargetException;

/**
 * This interface is implemented by all classes, that allow access to
 * fileds of an object.
 * 
 * @author wglas
 */
public interface FieldAccessor
{
    /**
     * @return The type of the field or property accessed.
     */
    public Class getType();
    
    /**
     * @return The class which contains the field or property accessed.
     */
    public Class getDeclaringClass();
    
    /**
     * @param type The annotation type to request for.
     * @return The annotation of the given field or property getter of the given type
     *         or null, if no such annotation exists.
     */
    public <T extends Annotation> T getAnnotation(Class<T> type);
    
    /**
     * @return All annotations of the given field or property getter
     *         or null, if no annotations exists.
     */
    public Annotation[] getAnnotations();
    
    /**
     * @return The name of the field or property accessed.
     */
    public String getName();

	/**
	 * Gets the associated field value of the given instance of the
	 * underlying class.
	 *
	 * @param pojo The object of which to get the field value.
	 * @return The value of the associated field.
	 */
	public Object getValueOf(Object pojo) throws IllegalArgumentException,
			IllegalAccessException, InvocationTargetException;

	/**
	 * Sets the associated field value on the given instance of the
	 * underlying class.
	 *
	 * @param pojo The object on which to set the field value.
	 * @param value The value of the associated field.
	 */
	public void setValueOn(Object pojo, Object value) throws IllegalArgumentException,
			IllegalAccessException, InvocationTargetException;
	
	/**
	 * Parse all relevant Annotations and try to retrieve a <code>Column</code>-Annotation and test it's <code>nullable()</code> value. 
	 * @return <code>true</code> for primitives,<br/><code>(!Column.nullable())</code> if a <code>Column</code>-Annotation could be found,<br/><code>false</code> otherwise. 
	 */
	public boolean checkMandatory();
	
    /**
     * Parse all relevant Annotations and try to retrieve a <code>Column</code>-Annotation and test it's <code>updatable()</code> value. 
     * @return <code>true</code> for primitives,<br/><code>Column.updatable()</code> if a <code>Column</code>-Annotation could be found,<br/><code>true</code> otherwise. 
     */
	public boolean checkUpdatable();
	
	/**
	 * Set the accessor used to access the ID-Field of the same POJO this <code>FieldAccessor</code> belongs to.
	 * @param idAccessor The <code>FieldAccessor</code> for the ID-Field.
	 */
	public void setIdAccessor(FieldAccessor idAccessor);
	
	/**
     * Get the accessor used to access the ID-Field of the same POJO this <code>FieldAccessor</code> belongs to.
     * @return The <code>FieldAccessor</code> for the ID-Field (may be null).
	 */
	public FieldAccessor getIdAccessor();

}
