/***********************************************************
 * $Id: ClasspathLSInput.java 79 2007-12-07 22:02:38Z wglas $
 * 
 * Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 2007-12-07
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 * 
 ***********************************************************/

package org.clazzes.util.xml;

import java.io.InputStream;
import java.io.Reader;

import org.w3c.dom.ls.LSInput;
import org.w3c.dom.ls.LSResourceResolver;

/**
 * This implementation of LSInput represents an XML input source
 * for classpath-resident schema or DTD descriptions, which are emitted
 * by a handcrafted instance of {@link LSResourceResolver}.
 * 
 * @author wglas
 */
public class ClasspathLSInput implements LSInput {

    private String resourcePath;
    private String baseURI;
    private String publicId;
    private String systemId;
    private boolean certifiedText;

    /**
     * Construct a classpath-resident LSInput instance. 
     * 
     * @param resourcePath The path, where the resource is located. Will be passed
     *                     to {@link ClassLoader#getResourceAsStream(String)}.
     * @param publicId The XML public ID of the resource.
     * @param systemId The XML system ID of the resource.
     * @param baseURI The XML base URI of the resource.
     */
    public ClasspathLSInput(String resourcePath,
            String publicId,
            String systemId,
            String baseURI) {
        
        this.resourcePath = resourcePath;
        this.publicId = publicId;
        this.systemId = systemId;
        this.baseURI = baseURI;
        this.certifiedText = true;
    }

    /**
     * Construct a classpath-resident LSInput instance. 
     * 
     * @param resourcePath The path, where the resource is located. Will be passed
     *                     to {@link ClassLoader#getResourceAsStream(String)}.
     * @param publicId The XML public ID of the resource.
     * @param systemId The XML system ID of the resource.
     * @param baseURI The XML base URI of the resource.
     * @param certifiedText The XML-1.1 flag, which marks the resource as certified. 
     */
    public ClasspathLSInput(String resourcePath,
            String publicId,
            String systemId,
            String baseURI,
            boolean certifiedText) {
        
        this.resourcePath = resourcePath;
        this.publicId = publicId;
        this.systemId = systemId;
        this.baseURI = baseURI;
        this.certifiedText = certifiedText;
    }

    /* (non-Javadoc)
     * @see org.w3c.dom.ls.LSInput#getBaseURI()
     */
    public String getBaseURI() {
        return this.baseURI;
    }

    /* (non-Javadoc)
     * @see org.w3c.dom.ls.LSInput#getByteStream()
     */
    public InputStream getByteStream() {
        InputStream is =
            ClasspathLSInput.class.getClassLoader().getResourceAsStream(this.resourcePath);
        
        return is;
    }

    /* (non-Javadoc)
     * @see org.w3c.dom.ls.LSInput#getCertifiedText()
     */
    public boolean getCertifiedText() {
        return this.certifiedText;
    }

    /* (non-Javadoc)
     * @see org.w3c.dom.ls.LSInput#getCharacterStream()
     */
    public Reader getCharacterStream() {
        return null;
    }

    /* (non-Javadoc)
     * @see org.w3c.dom.ls.LSInput#getEncoding()
     */
    public String getEncoding() {
        return null;
    }

    /* (non-Javadoc)
     * @see org.w3c.dom.ls.LSInput#getPublicId()
     */
    public String getPublicId() {
        return this.publicId;
    }

    /* (non-Javadoc)
     * @see org.w3c.dom.ls.LSInput#getStringData()
     */
    public String getStringData() {
        return null;
    }

    /* (non-Javadoc)
     * @see org.w3c.dom.ls.LSInput#getSystemId()
     */
    public String getSystemId() {
        return this.systemId;
    }

    /* (non-Javadoc)
     * @see org.w3c.dom.ls.LSInput#setBaseURI(java.lang.String)
     */
    public void setBaseURI(String baseURI) {
        this.baseURI = baseURI;
    }

    /* (non-Javadoc)
     * @see org.w3c.dom.ls.LSInput#setByteStream(java.io.InputStream)
     */
    public void setByteStream(InputStream byteStream) {
        throw new UnsupportedOperationException();
    }

    /* (non-Javadoc)
     * @see org.w3c.dom.ls.LSInput#setCertifiedText(boolean)
     */
    public void setCertifiedText(boolean certifiedText) {
        this.certifiedText = certifiedText;
    }

    /* (non-Javadoc)
     * @see org.w3c.dom.ls.LSInput#setCharacterStream(java.io.Reader)
     */
    public void setCharacterStream(Reader characterStream) {
        throw new UnsupportedOperationException();
    }

    /* (non-Javadoc)
     * @see org.w3c.dom.ls.LSInput#setEncoding(java.lang.String)
     */
    public void setEncoding(String encoding) {
        throw new UnsupportedOperationException();
    }

    /* (non-Javadoc)
     * @see org.w3c.dom.ls.LSInput#setPublicId(java.lang.String)
     */
    public void setPublicId(String publicId) {
        this.publicId = publicId;
    }

    /* (non-Javadoc)
     * @see org.w3c.dom.ls.LSInput#setStringData(java.lang.String)
     */
    public void setStringData(String stringData) {
        throw new UnsupportedOperationException();
    }

    /* (non-Javadoc)
     * @see org.w3c.dom.ls.LSInput#setSystemId(java.lang.String)
     */
    public void setSystemId(String systemId) {
        this.systemId = systemId;
    }


}
